public class BTree<E>  {
    private class Node {
        E elem;
        Node l;
        Node r;

        void addLeft(Node n) {
            l = n;
        }
        void addRight(Node n) {
            r = n;
        }
        Node(E elem) { this.elem = elem; }
    }

    private Node root = null;

    private BTree(Node n) {
        root = n;
    }

    /* ---------------------------------*/ 
    /* INTERFACE                        */
    /* ---------------------------------*/ 
    
    public BTree(E elem) {
        root = new Node(elem);
    }

    public BTree<E> addLeft(BTree<E> t) {
        root.addLeft(t.root);
        return this;
    }

    public BTree<E> addRight(BTree<E> t) {
        root.addRight(t.root);
        return this;
    }

    public BTree<E> linkToLeft(BTree<E> t) {
        t.root.addLeft(root);
        return this;
    }

    public BTree<E> linkToRight(BTree<E> t) {
        t.root.addRight(root);
        return this;
    }

    public BTree<E> getLeftSubtree() {
        if (root == null) return null;
        else return new BTree<E>(root.l);
    }

    public BTree<E> getRightSubtree() {
        if (root == null) return null;
        else return new BTree<E>(root.r);
    }

    void printPostOrder() {
        if (root == null) return;
        else {
            getLeftSubtree().printPostOrder();
            getRightSubtree().printPostOrder();
            System.out.print(root.elem);
            System.out.print(" ");
        }
    }

    void printPreOrder() {
        if (root == null) return;
        else {
            System.out.print(root.elem);
            System.out.print(" ");
            getLeftSubtree().printPreOrder();
            getRightSubtree().printPreOrder();
        }
    }

    void printInOrder() {
        if (root == null) return;
        else {
            getLeftSubtree().printInOrder();
            System.out.print(root.elem);
            System.out.print(" ");
            getRightSubtree().printInOrder();
        }
    }

    /* ---------------------------------*/ 
    /* TEST                             */
    /* ---------------------------------*/ 
    public static void main(String args[]) {
        BTree<String> mytree = new BTree<String>("*");
        
        BTree<String> ll = new BTree<String>("+");

        BTree<String> rr = new BTree<String>("-");

        ll.addLeft(new BTree<String>("2")).
            addRight(new BTree<String>("3")).
            linkToLeft(mytree);
        rr.addLeft(new BTree<String>("6")).
            addRight(new BTree<String>("4")).
            linkToRight(mytree);
        
        System.out.println("Post Order: ");
        mytree.printPostOrder();
        System.out.println("\nPre Order: ");
        mytree.printPreOrder();
        System.out.println("\nIn Order: ");
        mytree.printInOrder();
        System.out.println("\n");
    }  
}
